@@ -1,5 +1,6 @@
 import shutil
 import sys
+import tempfile
 
 from collections import OrderedDict
 from operator import itemgetter
@@ -26,17 +27,17 @@ class ScaleneOutput:
     cpu_percent_threshold = 1
 
     # Default threshold for number of mallocs to report a file.
-    malloc_threshold = 100
+    malloc_threshold = 1  # 100
 
-    def __init__(self):
+    def __init__(self) -> None:
         # where we write profile info
-        output_file: str = ""
+        self.output_file = ""
 
         # if we output HTML or not
-        html: bool = False
+        self.html = False
 
         # if we are on a GPU or not
-        gpu: bool = False
+        self.gpu = False
 
     # Profile output methods
     def output_profile_line(
@@ -111,20 +112,22 @@ class ScaleneOutput:
 
         if False:
             # Currently disabled; possibly use in another column?
-            # Correct for number of samples
+            # Normalize by number of samples ("net *average*")
             for bytei in stats.memory_malloc_count[fname][
                 line_no
             ]:  # type : ignore
-                n_malloc_mb /= stats.memory_malloc_count[fname][line_no][bytei]
-                n_python_malloc_mb /= stats.memory_malloc_count[fname][
-                    line_no
-                ][bytei]
+                count = stats.memory_malloc_count[fname][line_no][bytei]
+                if count > 0:
+                    n_malloc_mb /= count
+                    n_python_malloc_mb /= count
             for bytei in stats.memory_free_count[fname][line_no]:
-                n_free_mb /= stats.memory_free_count[fname][line_no][bytei]
+                count = stats.memory_free_count[fname][line_no][bytei]
+                if count > 0:
+                    n_free_mb /= count
 
         n_growth_mb = n_malloc_mb - n_free_mb
-        if -1 < n_growth_mb < 0:
-            # Don't print out "-0".
+        if -1 < n_growth_mb < 1:
+            # Don't print out "-0" or anything below 1.
             n_growth_mb = 0
 
         n_cpu_percent = n_cpu_percent_c + n_cpu_percent_python
@@ -135,7 +138,7 @@ class ScaleneOutput:
         # Adjust CPU time by utilization.
         n_cpu_percent_python *= stats.cpu_utilization[fname][line_no].mean()
         n_cpu_percent_c *= stats.cpu_utilization[fname][line_no].mean()
-        
+
         # Finally, print results.
         n_cpu_percent_c_str: str = (
             "" if n_cpu_percent_c < 1 else f"{n_cpu_percent_c:5.0f}%"
@@ -146,18 +149,17 @@ class ScaleneOutput:
         )
 
         n_cpu_percent_python_str: str = (
-            ""
-            if n_cpu_percent_python < 1
-            else f"{n_cpu_percent_python:5.0f}%"
+            "" if n_cpu_percent_python < 1 else f"{n_cpu_percent_python:5.0f}%"
         )
+        n_growth_mem_str = ""
         if n_growth_mb < 1024:
-            n_growth_mem_str: str = (
+            n_growth_mem_str = (
                 ""
                 if (not n_growth_mb and not n_usage_fraction)
-                else f"{n_growth_mb:5.1f}M"
+                else f"{n_growth_mb:5.0f}M"
             )
         else:
-            n_growth_mem_str: str = (
+            n_growth_mem_str = (
                 ""
                 if (not n_growth_mb and not n_usage_fraction)
                 else f"{(n_growth_mb / 1024):5.2f}G"
@@ -184,9 +186,9 @@ class ScaleneOutput:
         sys_str: str = (
             ""
             if n_sys_percent < 1
-            #or stats.cpu_utilization[fname][line_no].size() <= 1
-            #or stats.cpu_utilization[fname][line_no].sem() > 0.025
-            #or stats.cpu_utilization[fname][line_no].mean() > 0.99
+            # or stats.cpu_utilization[fname][line_no].size() <= 1
+            # or stats.cpu_utilization[fname][line_no].sem() > 0.025
+            # or stats.cpu_utilization[fname][line_no].mean() > 0.99
             else f"{n_sys_percent:4.0f}%"
         )
         if not is_function_summary:
@@ -373,7 +375,8 @@ class ScaleneOutput:
                         ),
                     )
 
-        null = open("/dev/null", "w")
+        null = tempfile.TemporaryFile(mode='w+')
+
         # Get column width of the terminal and adjust to fit.
         # Note that Scalene works best with at least 132 columns.
         column_width = 132
@@ -388,10 +391,7 @@ class ScaleneOutput:
                 pass
 
         console = Console(
-            width=column_width,
-            record=True,
-            force_terminal=True,
-            file=null,
+            width=column_width, record=True, force_terminal=True, file=null, force_jupyter=False,
         )
         # Build a list of files we will actually report on.
         report_files: List[Filename] = []
@@ -422,6 +422,9 @@ class ScaleneOutput:
             stats.output_stats(pid, python_alias_dir_name)
             return True
 
+        if len(report_files) == 0:
+            return False
+
         for fname in report_files:
 
             # If the file was actually a Jupyter (IPython) cell,
@@ -434,9 +437,12 @@ class ScaleneOutput:
                 fname_print = Filename("[" + result.group(1) + "]")
 
             # Print header.
-            percent_cpu_time = (
-                100 * stats.cpu_samples[fname] / stats.total_cpu_samples
-            )
+            if not stats.total_cpu_samples:
+                percent_cpu_time = 0
+            else:
+                percent_cpu_time = (
+                    100 * stats.cpu_samples[fname] / stats.total_cpu_samples
+                )
             new_title = mem_usage_line + (
                 "%s: %% of time = %6.2f%% out of %6.2fs."
                 % (fname_print, percent_cpu_time, stats.elapsed_time)
@@ -451,20 +457,40 @@ class ScaleneOutput:
                 width=column_width - 1,
             )
 
-            tbl.add_column("Line", style="dim", justify="right", no_wrap=True, width=4)
-            tbl.add_column(Markdown("Time  " + "\n" + "_Python_"), no_wrap=True, width=6)
-            tbl.add_column(Markdown("––––––  \n_native_"), no_wrap=True, width=6)
-            tbl.add_column(Markdown("––––––  \n_system_"), no_wrap=True, width=6)
+            tbl.add_column(
+                "Line", style="dim", justify="right", no_wrap=True, width=4
+            )
+            tbl.add_column(
+                Markdown("Time  " + "\n" + "_Python_"), no_wrap=True, width=6
+            )
+            tbl.add_column(
+                Markdown("––––––  \n_native_"), no_wrap=True, width=6
+            )
+            tbl.add_column(
+                Markdown("––––––  \n_system_"), no_wrap=True, width=6
+            )
             if self.gpu:
-                tbl.add_column(Markdown("––––––  \n_GPU_"), no_wrap=True, width=6)
+                tbl.add_column(
+                    Markdown("––––––  \n_GPU_"), no_wrap=True, width=6
+                )
 
             other_columns_width = 0  # Size taken up by all columns BUT code
 
             if profile_memory:
-                tbl.add_column(Markdown("Memory  \n_Python_"), no_wrap=True, width=7)
-                tbl.add_column(Markdown("––––––  \n_net_"), no_wrap=True, width=6)
-                tbl.add_column(Markdown("–––––––––––  \n_timeline_/%"), no_wrap=True, width=14)
-                tbl.add_column(Markdown("Copy  \n_(MB/s)_"), no_wrap=True, width=6)
+                tbl.add_column(
+                    Markdown("Memory  \n_Python_"), no_wrap=True, width=7
+                )
+                tbl.add_column(
+                    Markdown("––––––  \n_net_"), no_wrap=True, width=6
+                )
+                tbl.add_column(
+                    Markdown("–––––––––––  \n_timeline_/%"),
+                    no_wrap=True,
+                    width=14,
+                )
+                tbl.add_column(
+                    Markdown("Copy  \n_(MB/s)_"), no_wrap=True, width=6
+                )
                 other_columns_width = 75 + (6 if self.gpu else 0)
                 tbl.add_column(
                     "\n" + fname_print,
@@ -560,7 +586,6 @@ class ScaleneOutput:
                         tbl.add_row("", "", "", "", "", txt)
                     else:
                         tbl.add_row("", "", "", "", txt)
-                        
 
                 for fn_name in sorted(
                     fn_stats.cpu_samples_python,
